<?php
if (! class_exists('SLPTagalong_UI')) {
	require_once(SLPLUS_PLUGINDIR.'/include/base_class.userinterface.php');

    /**
     * Holds the UI-only code.
     *
     * This allows the main plugin to only include this file in the front end
     * via the wp_enqueue_scripts call.   Reduces the back-end footprint.
	 *
	 * @property		SLPTagalong						$addon
	 * @property-read	Tagalong_CategoryWalker_Legend 	$LegendWalker				Connect the legend walker object here.
     *
     * @package StoreLocatorPlus\Tagalong\UI
     * @author Lance Cleveland <lance@charlestonsw.com>
     * @copyright 2014 - 2015 Charleston Software Associates, LLC
     */
    class SLPTagalong_UI extends SLP_BaseClass_UI {
        public 	$addon;
	    private $LegendWalker;

        /**
         * Add WordPress and SLP hooks and filters.
         */
        public function add_hooks_and_filters() {
	        add_filter   ('slp_layout'                  ,array($this,'filter_AddLegend'              ),95    );
	        add_filter   ('slp_shortcode_atts'          ,array($this,'filter_SetAllowedShortcodes'   )       );
	        add_filter   ('shortcode_slp_searchelement' ,array($this,'filter_ProcessSearchElement'   )       );
	        add_filter   ('shortcode_storepage'         ,array($this,'filter_ProcessStorePage'       )       );
	        add_filter   ('slp_searchlayout'            ,array($this,'filter_ModifySearchLayout'     ),999   );
	        add_shortcode('tagalong'                    ,array($this,'process_TagalongShortcode'     )       );
        }

	    /**
	     * Setup the legend category walker object.
	     */
	    private function create_CategoryWalkerForLegend() {
		    if (class_exists('Tagalong_CategoryWalker_Legend') == false) {
			    require_once('class.categorywalker.legend.php');
		    }
		    if ( ! isset ( $this->LegendWalker ) ) {
			    $this->LegendWalker = new Tagalong_CategoryWalker_Legend(array('addon'=>$this->addon,'slplus'=>$this->slplus));
		    }
	    }

	    /**
	     * Add our custom category selection div to the search form.
	     *
	     * @return string the HTML for this div appended to the other HTML
	     */
	    private function createstring_CategorySelector() {

		    // Only With Category shortcode.
		    //
		    if ( ! empty( $this->slplus->options['only_with_category'] ) ) {
			    $category_list = array();
			    $category_slugs = preg_split( '/,/' , $this->slplus->options['only_with_category'] );
			    foreach ( $category_slugs as $slug ) {
				    $category = get_term_by('slug',sanitize_title( $slug ),SLPlus::locationTaxonomy);
				    if ( $category ) { $category_list[] = $category->term_id; }
			    }
			    $category_id_list = join( ',' , $category_list );
			    $this->addon->options['show_cats_on_search'] = 'only_with_category';
		    } else {
			    $category_id_list = '';
		    }

		    // Process the category selector type
		    //
		    switch ($this->addon->options['show_cats_on_search']) {

			    case 'only_with_category':
				    if ( ! empty( $category_id_list ) ) {
					    $HTML =
						    "<input type='hidden' name='cat' id='cat' ".
						    "value='{$category_id_list}' " .
						    "textvalue='{$this->slplus->options['only_with_category']}' " .
						    '/>';
				    } else {
					    $HTML = "<!-- only_with_category term {$this->slplus->options['only_with_category']}  does not exist -->";
				    }
				    break;

				// Cascade
			    // Single
				// None
				// DEFAULT
				case 'cascade':
				case 'none':
			    case 'single':
				default:
					$this->addon->create_object_category_selector_manager();
					$HTML = $this->addon->category_selector_manager->get_html( $this->addon->options['show_cats_on_search'] );
				    break;
		    }

		    return $HTML;
	    }
	    
	    /**
	     * Create the LegendHTML String.
	     *
	     * @return string
	     */
	    private function createstring_LegendHTML() {
		    $this->create_CategoryWalkerForLegend();

		    $HTML =
			    '<div id="tagalong_legend">'                        .
			    '<div id="tagalong_list">'                      .
			    wp_list_categories(
				    array(
					    'echo'              => 0,
					    'hierarchical'      => 0,
					    'depth'             => 99,
					    'hide_empty'        => ( $this->slplus->is_CheckTrue( $this->addon->options['hide_empty'] ) ? 1 : 0 ),
					    'style'             => 'none',
					    'taxonomy'          => 'stores',
					    'walker'            => $this->LegendWalker
				    )
			    ) .
			    '</div>'.
			    '</div>'
		    ;
		    return $HTML;
	    }


	    /**
	     * Add the Tagalong shortcode processing to whatever filter/hook we need it latched on to.
	     *
	     * The [tagalong] shortcode, used here, is setup in slp_init.
	     */
	    function filter_AddLegend($layoutString) {
		    return do_shortcode($layoutString);
	    }

	    /**
	     * Add Tagalong category selector to search layout.
	     *
	     */
	    function filter_ModifySearchLayout($layout) {
		    if ( ! empty ( $this->slplus->options['only_with_category'] ) ) { $this->addon->options['show_cats_on_search'] = '1'; }
		    if ( empty( $this->addon->options['show_cats_on_search'] ) ) { return $layout; }

		    if (preg_match('/\[slp_search_element\s+.*dropdown_with_label="category".*\]/i',$layout)) { return $layout; }
		    return $layout . '[slp_search_element dropdown_with_label="category"]';
	    }	    

	    /**
	     * Perform extra search form element processing.
	     *
	     * @param mixed[] $attributes
	     * @return mixed[]
	     */
	    function filter_ProcessSearchElement($attributes) {
		    foreach ($attributes as $name=>$value) {

			    switch (strtolower($name)) {

				    case 'selector_with_label':
				    case 'dropdown_with_label':
					    switch ($value) {
						    case 'category':
							    return array(
								    'hard_coded_value' =>
									    ! empty( $this->addon->options['show_cats_on_search'] )    ?
										    $this->createstring_CategorySelector()          :
										    ''
							    );
							    break;

						    default:
							    break;
					    }
					    break;

				    default:
					    break;
			    }
		    }

		    return $attributes;
	    }

	    /**
	     * Perform extra search form element processing.
	     *
	     * @param mixed[] $attributes
	     * @return mixed[]
	     */
	    function filter_ProcessStorePage($attributes) {
		    $this->addon->set_LocationCategories();

		    // No categories set?  Get outta here...
		    //
		    if ( count( $this->addon->current_location_categories ) < 1 ) { return $attributes; }

		    foreach ($attributes as $name=>$value) {

			    switch (strtolower($name)) {

				    case 'field':
					    switch ($value) {
						    case 'iconarray':
							    return array(
								    'hard_coded_value' => $this->addon->createstring_IconArray()
							    );
							    break;

						    default:
							    break;
					    }
					    break;

				    default:
					    break;
			    }
		    }

		    return $attributes;
	    }

	    /**
	     * Set the allowed shortcode attributes
	     *
	     * @param mixed[] $atts
	     * @return mixed[]
	     */
	    function filter_SetAllowedShortcodes($atts) {
		    return array_merge(
			    array(
				    'only_with_category'     => null,
			    ),
			    $atts
		    );
	    }

	    /**
	     * Process the Tagalong shortcode
	     */
	    function process_TagalongShortcode($atts) {
		    if (is_array($atts)) {
			    $theKeys = array_map('strtolower',$atts);
			    switch ($theKeys[0]) {
				    case 'legend':
					    return $this->createstring_LegendHTML();
					    break;

				    default:
					    break;
			    }
			    return '';
		    }
	    }	    
    }
}